package com.szyx.tcm.supervision.exception;


import com.fiftyonetrust.common.model.exception.CodeException;
import com.fiftyonetrust.common.model.exception.CodeRuntimeException;
import com.fiftyonetrust.common.model.result.R;
import com.fiftyonetrust.common.util.CommonConstants;
import lombok.extern.slf4j.Slf4j;
import org.dromara.hutool.core.text.StrUtil;
import org.springframework.http.converter.HttpMessageConversionException;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Slf4j
@RestControllerAdvice
public class GlabalExceptionHandler {
    //拦截Throwable异常
    @ExceptionHandler(value = Throwable.class)
    public R<String> throwableHandler(Throwable e, final HttpServletRequest request, final HttpServletResponse response) {
        log.error("系统异常", e);
        response.setHeader("Content-Type", CommonConstants.CONTENT_TYPE_JSON);
        return R.failed("系统异常");
    }

    //拦截全部异常
    @ExceptionHandler(value = Exception.class)
    public R<String> exceptionHandler(Exception e, final HttpServletRequest request, final HttpServletResponse response) {
        log.error("系统异常", e);
        response.setHeader("Content-Type", CommonConstants.CONTENT_TYPE_JSON);
        return R.failed("系统异常");
    }

    //拦截自定义异常
    @ExceptionHandler(value = CodeException.class)
    public R<String> codeExceptionHandler(CodeException e, final HttpServletRequest request, final HttpServletResponse response) {
        log.warn("CodeException异常, {}, {}", e.getCode(), e.getMessage());
        response.setHeader("Content-Type", CommonConstants.CONTENT_TYPE_JSON);
        return R.failed(e);
    }

    //拦截自定义异常
    @ExceptionHandler(value = CodeRuntimeException.class)
    public R<String> codeRuntimeExceptionHandler(CodeRuntimeException e, final HttpServletRequest request, final HttpServletResponse response) {
        log.error("CodeRuntimeException异常, {}, {}", e.getCode(), e.getMessage(), e);
        response.setHeader("Content-Type", CommonConstants.CONTENT_TYPE_JSON);
        return R.failed(e);
    }

    /**
     * 校验错误拦截处理
     *
     * @param exception 错误信息集合
     * @return 错误信息
     */
    @ExceptionHandler({MethodArgumentNotValidException.class})
    public R<String> validationBodyException(MethodArgumentNotValidException exception) {
        return validationBindingResult(exception.getBindingResult());
    }

    @ExceptionHandler({BindException.class})
    public R<String> validationBodyException(BindException exception) {
        return validationBindingResult(exception.getBindingResult());
    }
    /**
     * 参数类型转换错误
     *
     * @param exception 错误
     * @return 错误信息
     */
    @ExceptionHandler(HttpMessageConversionException.class)
    public R<String> parameterTypeException(HttpMessageConversionException exception) {
        log.error(exception.getCause().getLocalizedMessage());
        return R.failed("类型转换错误");
    }

    private R<String> validationBindingResult(BindingResult result){
        String message = "";
        if (result.hasErrors()) {
            List<ObjectError> errors = result.getAllErrors();
            if (errors != null) {
                errors.forEach(p -> {
                    FieldError fieldError = (FieldError) p;
                    log.error("Data check failure : object{" + fieldError.getObjectName() + "},field{" + fieldError.getField() +
                            "},errorMessage{" + fieldError.getDefaultMessage() + "}");

                });
                if (errors.size() > 0) {
                    FieldError fieldError = (FieldError) errors.get(0);
                    message = fieldError.getDefaultMessage();
                }
            }
        }
        return R.failed(StrUtil.isBlank(message) ? "请填写正确信息" : message);
    }
}

